/*
  These functions are very deprecated and should not be used further.
*/

const char       * buffer_fread_string(buffer_type * buffer);
char             * buffer_fread_alloc_string(buffer_type * buffer);
void               buffer_fwrite_string(buffer_type * buffer , const char * string);
