from ecl.well import WellTypeEnum
from ecl.well import WellConnectionDirectionEnum
from ecl.well import WellConnection
from ecl.well import WellSegment
from ecl.well import WellState
from ecl.well import WellTimeLine
from ecl.well import WellInfo
