/*
   Copyright (C) 2017  Statoil ASA, Norway.

   The file 'util_unlink.c' is part of ERT - Ensemble based Reservoir Tool.

   ERT is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   ERT is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU General Public License at <http://www.gnu.org/licenses/gpl.html>
   for more details.
*/

#include "ert/util/build_config.hpp"

#include <ert/util/util.h>

#if defined(HAVE_WINDOWS_UNLINK)

#include <io.h>

int util_unlink(const char * filename) {
 return _unlink(filename);
}

#elif defined(HAVE_POSIX_UNLINK)

#include <unistd.h>

int util_unlink(const char * filename) {
 return unlink(filename);
}

#else

#error "No unlinke implementation on this platform"

#endif
