/*
  Warning: The libecl code has changed to be compiled as a C++ project. This
  header file is retained for a period for compatibility, but you are encouraged
  to switch to include the new hpp header directly in your code.
*/

#include <ert/ecl/ecl_rft_cell.hpp>


